/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2007-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://socialsite.dev.java.net/legal/CDDL+GPL.html
 * or legal/LICENSE.txt.  See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at legal/LICENSE.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided by Sun
 * in the GPL Version 2 section of the License file that accompanied this code.
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.socialsite.web.rest.model;

import com.sun.socialsite.pojos.AppRegistration;
import java.util.Date;
import org.apache.shindig.gadgets.oauth.BasicOAuthStoreConsumerKeyAndSecret.KeyType;


/**
 * Simple model object with bare minimum app registration fields.
 */
public class AppRegistrationWrapper {
    private String id = null;
    private String ownerId = null;
    private String appUrl = null;
    private String status = null;
    private Date created = null;
    private Date updated = null;

    // key and secret so Gadgests's server can call SocailSite REST API
    // generated by SocialSite
    private String consumerKey = null;
    private String consumerSecret = null;

    // name, key and secret for calling Gadget's server
    // generated by Gadget's server
    private String serviceName = null;
    private String serviceConsumerKey = null;
    private String serviceConsumerSecret = null;
    private String serviceKeyType = null;

    private String comment = null;

    public AppRegistrationWrapper() {
        super();
    }

    public AppRegistrationWrapper(AppRegistration appReg) {
        super();
        this.id = appReg.getId();
        this.serviceName = appReg.getServiceName();
        this.appUrl = appReg.getAppUrl();
        this.ownerId = appReg.getProfile().getUserId();
        this.created = new Date(appReg.getCreated().getTime());
        this.updated = new Date(appReg.getUpdated().getTime());
        this.status = appReg.getStatus();
        this.consumerSecret = appReg.getConsumerSecret();
        this.consumerKey = appReg.getConsumerKey();
        this.comment = appReg.getComment();
    }

    public AppRegistrationWrapper(AppRegistration appReg,
            String serviceConsumerKey,
            String serviceConsumerSecret,
            String serviceKeyType) { 
        this(appReg);
        this.serviceConsumerKey = serviceConsumerKey;
        this.serviceConsumerSecret = serviceConsumerSecret;
        this.serviceKeyType = serviceKeyType;
    }

    /**
     * @return the id
     */
    public String getId() {
        return id;
    }

    /**
     * @param id the id to set
     */
    public void setId(String id) {
        this.id = id;
    }

    /**
     * @return the ownerId
     */
    public String getOwnerId() {
        return ownerId;
    }

    /**
     * @param ownerId the ownerId to set
     */
    public void setOwnerId(String ownerId) {
        this.ownerId = ownerId;
    }

    /**
     * @return the appUrl
     */
    public String getAppUrl() {
        return appUrl;
    }

    /**
     * @param appUrl the appUrl to set
     */
    public void setAppUrl(String appUrl) {
        this.appUrl = appUrl;
    }

    /**
     * @return the created
     */
    public Date getCreated() {
        return created;
    }

    /**
     * @param created the created to set
     */
    public void setCreated(Date created) {
        this.created = created;
    }

    /**
     * @return the updated
     */
    public Date getUpdated() {
        return updated;
    }

    /**
     * @param updated the updated to set
     */
    public void setUpdated(Date updated) {
        this.updated = updated;
    }

    @Override
    public boolean equals(Object o) {
        AppRegistrationWrapper other = (AppRegistrationWrapper)o;
        return getAppUrl().equals(other.getAppUrl());
    }

    /**
     * @return the status
     */
    public String getStatus() {
        return status;
    }

    /**
     * @param status the status to set
     */
    public void setStatus(String status) {
        this.status = status;
    }

    /**
     * @return the consumerSecret
     */
    public String getConsumerSecret() {
        return consumerSecret;
    }

    /**
     * @param consumerSecret the consumerSecret to set
     */
    public void setConsumerSecret(String consumerSecret) {
        this.consumerSecret = consumerSecret;
    }

    /**
     * @return the comment
     */
    public String getComment() {
        return comment;
    }

    /**
     * @param comment the comment to set
     */
    public void setComment(String comment) {
        this.comment = comment;
    }

    /**
     * @return the consumerKey
     */
    public String getConsumerKey() {
        return consumerKey;
    }

    /**
     * @param consumerKey the consumerKey to set
     */
    public void setConsumerKey(String consumerKey) {
        this.consumerKey = consumerKey;
    }

    /**
     * @return the serviceName
     */
    public String getServiceName() {
        return serviceName;
    }

    /**
     * @param serviceName the serviceName to set
     */
    public void setServiceName(String serviceName) {
        this.serviceName = serviceName;
    }

    /**
     * @return the serviceConsumerKey
     */
    public String getServiceConsumerKey() {
        return serviceConsumerKey;
    }

    /**
     * @param serviceConsumerKey the serviceConsumerKey to set
     */
    public void setServiceConsumerKey(String serviceConsumerKey) {
        this.serviceConsumerKey = serviceConsumerKey;
    }

    /**
     * @return the serviceConsumerSecret
     */
    public String getServiceConsumerSecret() {
        return serviceConsumerSecret;
    }

    /**
     * @param serviceConsumerSecret the serviceConsumerSecret to set
     */
    public void setServiceConsumerSecret(String serviceConsumerSecret) {
        this.serviceConsumerSecret = serviceConsumerSecret;
    }

    /**
     * @return the serviceKeyType
     */
    public String getServiceKeyType() {
        return serviceKeyType;
    }

    /**
     * @param serviceKeyType the serviceKeyType to set
     */
    public void setServiceKeyType(String serviceKeyType) {
        this.serviceKeyType = serviceKeyType;
    }

}
